import os
import triton
import triton.language as tl

def _test_pid_conds(conds, pid0=0, pid1=0, pid2=0):
    '''Test if condition on pids are fulfilled
    E.g.:
        '=0'    checks that pid_0 == 0
        ',>1'   checks that pid_1 > 1
        '>1,=0' checks that pid_0 > 1 and pid_1 == 0
    '''
    pids = pid0, pid1, pid2
    conds = conds.replace(' ','').split(',')
    for i, (cond, pid) in enumerate(zip(conds, pids)):
        if cond=='': continue
        if   cond[:2] in ['<=', '>=', '!=']: op, threshold = cond[:2], int(cond[2:])
        elif cond[:1] in ['<',  '>',  '=' ]: op, threshold = cond[:1], int(cond[1:])        
        else: raise ValueError(f"Rules may only use these ops: '<','>','>=','<=','=', '!='. Invalid rule in '{cond}'.")
        op = '==' if op == '=' else op
        if not eval(f'{pid} {op} {threshold}'): return False
    return True

@triton.jit
def test_pid_conds(conds):
    '''Test if condition on pids are fulfilled
    E.g.:
        '=0'    checks that pid_0 == 0
        ',>1'   checks that pid_1 > 1
        '>1,=0' checks that pid_0 > 1 and pid_1 == 0
    '''
    return _test_pid_conds(tl.program_id(0)[0], tl.program_id(1)[0], tl.program_id(2)[0])

@triton.jit
def breakpoint_if(conds):
    '''Stop kernel, if condition on pids is fulfilled'''
    from IPython.core.debugger import set_trace
    if test_pid_conds(conds): set_trace()

@triton.jit
def print_if(txt, conds):
    '''Print txt, if condition on pids is fulfilled'''
    if test_pid_conds(conds): print(txt)

@triton.jit
def breakpoint_once(): breakpoint_if('=0,=0,=0')

@triton.jit
def print_once(txt): print_if(txt,'=0,=0,=0')

def assert_tensors_gpu_ready(*tensors):
    for t in tensors:
        assert t.is_contiguous(), "A tensor is not contiguous"
        if not os.environ.get('TRITON_INTERPRET') == '1': assert t.is_cuda, "A tensor is not on cuda"




import unittest

from triton_util.debugging import _test_pid_conds

class TestTritonUtil(unittest.TestCase):
    def test_test_pid_conds(self):
        self.assertTrue(_test_pid_conds(''))
        self.assertTrue(_test_pid_conds('>0', 1, 1))
        self.assertFalse(_test_pid_conds('>0', 0, 1))
        self.assertTrue(_test_pid_conds('=0,=1', 0, 1, 0))

if __name__ == '__main__':
    unittest.main()



import triton
import triton.language as tl

def cdiv(a,b): return (a + b - 1) // b

@triton.jit
def get_1d_offset(sz, n_prev_chunks=0): return n_prev_chunks * sz + tl.arange(0, sz)

@triton.jit
def get_2d_offset(offs0, offs1, stride0, stride1=1):  return tl.expand_dims(offs0, 1)*stride0 + tl.expand_dims(offs1, 0)*stride1

@triton.jit
def get_1d_mask(offs, max): return offs < max

@triton.jit
def get_2d_mask(offs0, offs1, max0, max1): return (tl.expand_dims(offs0, 1) < max0) & (tl.expand_dims(offs1, 0) < max1)

@triton.jit
def load_2d(ptr, sz0, sz1, n0, n1, max0, max1, stride0, stride1=1):
    '''Chunk 2d matrix (defined by ptr) into 2d grid, where each chunk has size (sz0,sz1). Load the (n0,n1)th chunk. Ie, load [n0*sz0,...,(n0+1)*sz0-1] x [n1*sz1,...,(n1+1)*sz1-1].'''
    offs0 = get_1d_offset(sz0, n0)
    offs1 = get_1d_offset(sz1, n1)        
    offs = get_2d_offset(offs0, offs1, stride0, stride1)
    mask = get_2d_mask(offs0, offs1, max0, max1)
    return tl.load(ptr + offs, mask) 

@triton.jit
def load_full_2d(ptr, sz0, sz1, stride0, stride1=1):
    '''Load 2d block [0,...,sz0-1] x [0,...,sz1-1] '''
    offs = get_2d_offset(tl.arange(0, sz0), tl.arange(0, sz1), stride0, stride1)
    mask = get_2d_mask(  tl.arange(0, sz0), tl.arange(0, sz1), sz0, sz1)
    return tl.load(ptr + offs, mask) 

@triton.jit
def load_full_1d(ptr, sz, stride=1):
    '''Load 1d block [0,...,sz-1]'''
    offs = get_1d_offset(sz)
    mask = get_1d_mask(offs, sz)
    return tl.load(ptr + offs, mask) 




import unittest

from triton_util.coding import cdiv

class TestTritonUtil(unittest.TestCase):
    
    def test_cdiv(self):
        self.assertEqual(cdiv(10, 2), 5)
        self.assertEqual(cdiv(10, 3), 4)

if __name__ == '__main__':
    unittest.main()



from setuptools import setup, find_packages

setup(
    name='triton-util',
    version='0.0.2',
    packages=find_packages(),
    install_requires=['triton'],
    author='Umer Adil',
    author_email='umer.hayat.adil@gmail.com',
    description='Make Triton easier - A utility package for OpenAI Triton',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/umerHA/triton_util',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)
